/*******************************************************************************
 * @file    main.c
 * @author  Ahmed Eldeep
 * @email   ahmed@almohandes.org
 * @website http://almohandes.org/stm32
 * @date    21.03.2018
 *          
 * @brief   main application called after startup
 * @note    
 *
@verbatim
Copyright (C) Almohandes.org, 2018

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
@endverbatim
*******************************************************************************/

/* Includes */
#include "stm32f4xx.h"
#include "gpio.h"
#include "itm.h"

#include "rtos.h"



/**
 * @addtogroup stm32_examples
 * @{
 */

/**
 * @defgroup main
 * @brief
 * @{
 */

/**
 * @defgroup main_private_typedefs
 * @{
 */

/**
 * @}
 */

/**
 * @defgroup main_private_defines
 * @{
 */

/**
 * @}
 */

/**
 * @defgroup main_private_macros
 * @{
 */

/**
 * @}
 */

/**
 * @defgroup main_private_constants
 * @{
 */

/**
 * @}
 */

/**
 * @defgroup main_private_variables
 * @{
 */

static RTOS_thread_t thread1;
static RTOS_stack_t thread1stack;
static RTOS_thread_t thread2;
static RTOS_stack_t thread2stack;
static RTOS_thread_t thread3;
static RTOS_stack_t thread3stack;
static RTOS_thread_t thread4;
static RTOS_stack_t thread4stack;

static RTOS_mutex_t mutex1;
static RTOS_semaphore_t semaphore1;
static RTOS_mailbox_t mailbox1;
static uint32_t mailbox1buffer[3];

/**
 * @}
 */

/**
 * @defgroup main_private_function_prototypes
 * @{
 */

/**
 * @}
 */

/**
 * @defgroup main_private_functions
 * @{
 */
/**
 * @brief   thread1function
 * @note
 * @param   none
 * @retval  none
 */
void thread1function(void)
{
  uint32_t msg = 0x11223344;

  while(1)
  {
//    RTOS_SVC_semaphoreTake(&semaphore1, 1);
//
//    ITM_Printf("Hello World! from thread 1\n");
//    ITM_Printf("**************************\n");
//
//    RTOS_SVC_semaphoreGive(&semaphore1);

    RTOS_SVC_mailboxWrite(&mailbox1, 1, &msg);

    for (int var = 0; var < 1000000; ++var)
    {
    }
  }
}

/**
 * @brief   thread2function
 * @note
 * @param   none
 * @retval  none
 */
void thread2function(void)
{
  uint32_t msg = 0x55667788;

  while(1)
  {
//    RTOS_SVC_semaphoreTake(&semaphore1, 1);
//
//    ITM_Printf("Hello World! from thread 2\n");
//    ITM_Printf("**************************\n");
//
//    RTOS_SVC_semaphoreGive(&semaphore1);

    RTOS_SVC_mailboxWrite(&mailbox1, 1, &msg);

    for (int var = 0; var < 1000000; ++var)
    {
    }
  }
}

/**
 * @brief   thread2function
 * @note
 * @param   none
 * @retval  none
 */
void thread3function(void)
{
  uint32_t msg = 0x99101011;

  while(1)
  {
//    RTOS_SVC_semaphoreTake(&semaphore1, 1);
//
//    ITM_Printf("Hello World! from thread 3\n");
//    ITM_Printf("**************************\n");
//
//    RTOS_SVC_semaphoreGive(&semaphore1);

    RTOS_SVC_mailboxWrite(&mailbox1, 1, &msg);

    for (int var = 0; var < 1000000; ++var)
    {
    }
  }
}

/**
 * @brief   thread2function
 * @note
 * @param   none
 * @retval  none
 */
void thread4function(void)
{
  uint32_t msg;

  while(1)
  {
//    RTOS_SVC_semaphoreTake(&semaphore1, 1);
//
//    ITM_Printf("Hello World! from thread 4\n");
//    ITM_Printf("**************************\n");
//
//    RTOS_SVC_semaphoreGive(&semaphore1);

    RTOS_SVC_mailboxRead(&mailbox1, 1, &msg);

    if(0x11223344 == msg)
    {
      ITM_Printf("Message from thread 1\n");
    }
    else if(0x55667788 == msg)
    {
      ITM_Printf("Message from thread 2\n");
    }
    else if(0x99101011 == msg)
    {
      ITM_Printf("Message from thread 3\n");
    }
    else
    {
      /* Do nothing */
    }

    RTOS_SVC_mailboxRead(&mailbox1, 1, &msg);

    if(0x11223344 == msg)
    {
      ITM_Printf("Message from thread 1\n");
    }
    else if(0x55667788 == msg)
    {
      ITM_Printf("Message from thread 2\n");
    }
    else if(0x99101011 == msg)
    {
      ITM_Printf("Message from thread 3\n");
    }
    else
    {
      /* Do nothing */
    }

    for (int var = 0; var < 1000000; ++var)
    {
    }
  }
}

/**
 * @}
 */

/**
 * @defgroup main_exported_functions
 * @{
 */

/**
 * @brief   Main function
 * @note
 * @param   none
 * @retval  none
 */
int main(void)
{
  RTOS_init();

  RTOS_SVC_threadCreate(&thread1, &thread1stack, 1, thread1function);
  RTOS_SVC_threadCreate(&thread2, &thread2stack, 1, thread2function);
  RTOS_SVC_threadCreate(&thread3, &thread3stack, 1, thread3function);
  RTOS_SVC_threadCreate(&thread4, &thread4stack, 1, thread4function);


  RTOS_SVC_mutexCreate(&mutex1, 1);
  RTOS_SVC_semaphoreCreate(&semaphore1, 1);
  RTOS_SVC_mailboxCreate(&mailbox1, mailbox1buffer, 2, 4);


  RTOS_SVC_schedulerStart();

  /* Infinite loop */
  while(1)
  {

  }

}

/**
 * @}
 */
/**
 * @}
 */
/**
 * @}
 */
